/* 
    file copy program with unbuffered I/O.
    check with with `diff -s` from original and new file
*/
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>

#define BUFF_LEN 128

int main(int argc, char *argv[]) {
    if (argc != 3) {
        fprintf(stderr, "usage: %s <source_file> <destination_file>\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    // open a source file
    int src_fd = open(argv[1], O_RDONLY);
    if (src_fd < 0) {
        perror("error opening source file");
        exit(EXIT_FAILURE);
    }

    // open a destination file
    int dest_fd = open(argv[2], O_WRONLY | O_CREAT | O_TRUNC, 0644);
    if (dest_fd < 0) {
        perror("error opening destination file");
        close(src_fd);
        return EXIT_FAILURE;
    }

    char buffer[BUFF_LEN];
    ssize_t bytes_read, bytes_written, ret;

    // read the source file in chunks of 128 bytes
    while ((bytes_read = read(src_fd, buffer, BUFF_LEN)) > 0) {
        bytes_written = 0;
        // write bytes to the destination file
        while((ret = write(dest_fd, buffer, bytes_read - bytes_written)) > 0) {
            bytes_written += ret;
        }
        // check the return value for errors
        if (bytes_written != bytes_read || ret < 0) {
            perror("error writing to destination file");
            // close files and exit
            close(src_fd);
            close(dest_fd);
            exit(EXIT_FAILURE);
        }
    }

    if (bytes_read < 0) {
        perror("error reading source file");
    }

    // close both files
    close(src_fd);
    close(dest_fd);

    return EXIT_SUCCESS;
}
