/* double check work with `wc -wl` */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#define BUFF_LEN 1024

// count the words in a '\0' terminated string
int count_words(const char *line) {
    int count = 0, word_found = 0;
    while (*line) {
        if (!isspace((char)*line)) {
            // word was found, mark start
            word_found = 1;
        } else if (word_found) {
            // whitespace was found, reset flag
            word_found = 0;
            // count the completed word
            count++;
        }
        line++;
    }
    return count;
}

int main(int argc, char *argv[]) {
    if (argc != 2) {
        fprintf(stderr, "usage: %s <filename>\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    // 1. open a text file
    FILE *file = fopen(argv[1], "r");
    if (file == NULL) {
        perror("error opening file");
        exit(EXIT_FAILURE);
    }

    char buffer[BUFF_LEN];
    int line_num = 1, num_words = 0, total_words = 0;
    
    // 2. read it line by line
    while (fgets(buffer, BUFF_LEN, file)) {
        // 3. for each line, count the number of words and print
        num_words = count_words(buffer);
        printf("line %d: %d words\n", line_num++, num_words);
        total_words += num_words;
    }
    printf("%d total words\n", total_words);

    // 4. close the file
    int ret = fclose(file);
    if (ret != 0) {
        perror("closing file");
        exit(EXIT_FAILURE);
    }

    return EXIT_SUCCESS;
}
