/* implements grep -o <char> <file> funtionality */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>

int main(int argc, char *argv[]) {
    if (argc != 3) {
        fprintf(stderr, "usage: %s <character> <filename>\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    char *character = argv[1];
    char *filename = argv[2];

    if (character[1] != '\0') {
        fprintf(stderr, "error: only input a single character\n");
        exit(EXIT_FAILURE);
    }

    pid_t pid = fork();

    if (pid < 0) {
        perror("fork failed");
        exit(EXIT_FAILURE);
    } else if (pid == 0) {
        // In child process
        // use execl to run `grep -o <char> <filename>`
	execl(
            "/usr/bin/grep",
            "/usr/bin/grep",
            "-o",
            character,
            filename,
            (char *) NULL);
        perror("execl failed");
        exit(EXIT_FAILURE);
    } else {
        // In parent process
        int status;
	// wait for child process to finish
        waitpid(pid, &status, 0);
	// check if child's return status was non-zero (error)
        if (WIFEXITED(status)) {
	    // get child's return code from status
            int exit_status = WEXITSTATUS(status);
            return exit_status;
        }
    }

    return 0;
}
