#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <sys/wait.h>

#define BUFFER_SIZE 100
#define READ_END 0
#define WRITE_END 1

// convert string to uppercase
// modifies null-terminated string in place
void to_uppercase(char *str) {
    int str_len = strlen(str);
    for (int i = 0; i < str_len; i++) {
        str[i] = toupper(str[i]);
    }
}

int main() {
    pid_t pid;
    // parent to child pipe
    int parent_pipe[2];
    // child to parent pipe
    int child_pipe[2];
    // message to be sent to parent
    char message[] = "Hello from parent!";
    char buffer[BUFFER_SIZE];
    // initialize buffer values to NULL
    memset(buffer, 0, BUFFER_SIZE);

    // create the pipes
    if (pipe(parent_pipe) == -1 || pipe(child_pipe) == -1) {
        perror("pipe");
        exit(EXIT_FAILURE);
    }

    // create child process
    pid = fork();

    if (pid < 0) {
        perror("fork");
        exit(EXIT_FAILURE);
    }

    if (pid == 0) {
        // in child process

        // close unused ends
        close(parent_pipe[WRITE_END]); // close write end of parent_pipe
        close(child_pipe[READ_END]); // close read end of child_pipe

        // read from parent
        read(parent_pipe[READ_END], buffer, BUFFER_SIZE);
        close(parent_pipe[READ_END]);

        // modify parent's message
        to_uppercase(buffer);

        // send modified message to parent
        write(child_pipe[WRITE_END], buffer, strlen(buffer) + 1);
        close(child_pipe[WRITE_END]);

    } else {
        // in parent process

        // close unused ends
        close(parent_pipe[READ_END]); // close read end of parent_pipe
        close(child_pipe[WRITE_END]); // close write end of child_pipe

        // send message to child
        write(parent_pipe[WRITE_END], message, strlen(message) + 1);
        close(parent_pipe[WRITE_END]);

        // read modified message from child
        read(child_pipe[READ_END], buffer, BUFFER_SIZE);
        close(child_pipe[READ_END]);

        // print out message contents
        printf("original message: %s\n", message);
        printf("modified message: %s\n", buffer);

        // wait for child to finish
        int status;
        waitpid(pid, &status, 0);
    }

    return 0;
}
